%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Technical Note TN114
% Parameters for 'Motor speed control' system
% Copyright imperix ltd, 2021
% dev@imperix.ch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% =================== System parameters ======================== %%

ControlFreq    = 20e3;              % Control interrupt frequency [Hz]
ControlPeriod  = 1/ControlFreq;     % Control interrupt period [s]
SamplingPeriod = ControlPeriod;     % Sampling period [s]

% Deadtime
dt = 1e-6;


%% =================== Plant model ======================== %%

% PMSM
Pmsm.p   = 3;			% Pole pairs [-]
Pmsm.Usn = 460;			% Rated voltage [V]
Pmsm.Isn = 2.43;		% Rated current [Arms]
Pmsm.fsn = 50;			% Rated frequency [Hz]
Pmsm.wsn = 2*pi*50;		% Rated angular frequency [rad/s]
Pmsm.Rs  = 3.4;			% Stator resistance [Ohm]
Pmsm.Ls  = 12.15e-3;	% Stator inductance [H]
Pmsm.Ld  = Pmsm.Ls;     % Stator inductance along d axis [H]
Pmsm.Lq  = Pmsm.Ls;     % Stator inductance along q axis [H]

Pmsm.Jm  = 2.9e-4;		% Moment of inertia [Nm^2]
Pmsm.kF  = 0.0003035;	% Friction coefficient 
Pmsm.Phi_pm = 0.25;		% Flux from permanent magnets [Nm/Apeak]

Pmsm.Temn = 3.9; 		% Nominal torque [Nm]
Pmsm.Pn   = 1.23e3;		% Nominal active power [W]

Pmsm.Is_max = 8.1;		% Peak current [A]
Pmsm.Wm_Max = 4800;		% Max mechanical speed [rpm]

% Encoder parameters
PPR     = 4096;         % Encoder pulses per turn [-]


%% ============== DC link - Voltage control =============== %%

DcLink.Rbr  = 50;		% Braking chopper resistor [Ohm]
DcLink.Vdc0 = 800;      % Initial DC bus voltage [V]


%% =============== PMSM - Current control ================= %%

% Plant model
Ipmsm.K1 = 1 / Pmsm.Rs;
Ipmsm.T1 = Pmsm.Ld / Pmsm.Rs;
Ipmsm.K2 = 1 / Pmsm.Rs;
Ipmsm.T2 = Pmsm.Lq / Pmsm.Rs;

% Small time constants
Ipmsm.Ts = ControlPeriod;           % Current controller control period [s]

Ipmsm.Tsens = 0;					% Sensing delay (ideal sensor) [s]
Ipmsm.Tctrl = 0.5 * Ipmsm.Ts;		% Control delay [s]
Ipmsm.Tpwm  = 0.5 * ControlPeriod;	% Modulator delay [s]

Ipmsm.Kcm  = 1;
Ipmsm.Ttot = Ipmsm.Tctrl + Ipmsm.Tpwm + Ipmsm.Tsens;

% Magnitude optimum
Ipmsm.Tn_d = Ipmsm.T1;
Ipmsm.Ti_d = 2 * Ipmsm.Kcm * Ipmsm.K1 * Ipmsm.Ttot;
Ipmsm.Tn_q = Ipmsm.T2;
Ipmsm.Ti_q = 2 * Ipmsm.Kcm * Ipmsm.K2 * Ipmsm.Ttot;

Ipmsm.Kp_d = Ipmsm.Tn_d / Ipmsm.Ti_d;
Ipmsm.Ki_d = 1 / Ipmsm.Ti_d;
Ipmsm.Kp_q = Ipmsm.Tn_q / Ipmsm.Ti_q;
Ipmsm.Ki_q = 1 / Ipmsm.Ti_q;


%% =============== PMSM - Speed control =================== %%

% Plant model
Speed.T1 = Pmsm.Jm;

% Speed LPF cut-off frequency
Speed.fc = 1e3;

% Small time constants
Speed.Ratio = 100;                  % Decimation ratio of the speed control loop [s]
Speed.Ts = Speed.Ratio * Ipmsm.Ts;  % Speed controller period [s]

Speed.Tsens = ControlPeriod + 1/(2*pi*Speed.fc); % Sensing delay [s]
                                    % (angle derivative computation and LPF)
Speed.Tctrl = Speed.Ts;             % Control delay [s]

Speed.Kcm  = 1;
Speed.Ttot = Speed.Tctrl + Speed.Tsens;

% Symmetrical optimum
Speed.Tn = 4 * Speed.Ttot;
Speed.Ti = 8 * (Speed.Ttot^2 / Speed.T1) * Speed.Kcm;

Speed.Kp = Speed.Tn / Speed.Ti;
Speed.Ki = 1 / Speed.Ti;

% Torque saturation
Speed.Tem_sat = 1.1 * Pmsm.Temn;

